unit selfmod;

 { Author:  Trevor J Carlsen                                                 }
 {          PO Box 568                                                       }
 {          Port Hedland                                                     }
 {          Western Australia 6721                                           }
 {          Phone +61 91 73 2026 (voice)                                     }
 {                +61 91 73 2569 (data )                                     }
 { Copyright 1991 Trevor J Carlsen.                                          }
 { This program may be used as if it were in the public domain EXCEPT...     }
 {   1. Any program using the code must display a copyright notice as        }
 {      follows:  " Parts copyright 1991 Trevor J Carlsen"                   }
 {   2. The code remains unaltered (including these comments) if distributed }
 {      or copied.  If you do alter the code and wish to distribute it, a    }
 {      copy should first be sent to the above address.                      }
 { Allows a program to self modify a typed constant in the .exe file.  It    }
 { also performs an automatic checksum type .exe file integrity check.       }
 { A longint value is added to the end of the exe file.  This can be read by }
 { a separate configuration program to enable it to determine the start of   }
 { the programs configuration data area.  To use this the configuration      }
 { typed constant should be added immediately following the declaration of   }
 { ExeData.                                                                  }
 
 { Where this unit is used, it should always be the FIRST unit listed in the }
 { uses declaration area of the main program.                                }
 
 { Requires DOS 3.3 or later.  Program must not be used with PKLite or LZExe }
 { or any similar exe file compression programs.                             }
 
 { The stack size needed is at least 9,000 bytes.                            }
 
interface

type
  ExeDatatype    = record
                     IDStr      : string[8];
                     FirstTime  : boolean;
                     NumbExecs  : byte;
                     Hsize      : word;
                     ExeSize    : longint;
                     CheckSum   : longint;
                     StartConst : longint;
                   end;
                   
const
  ExeData : ExeDatatype = (IDStr     : 'IDString';
                           FirstTime : true;
                           NumbExecs : 0;
                           Hsize     : 0;
                           ExeSize   : 0;
                           CheckSum  : 0;
                           StartConst: 0);

{ IMPORTANT: Put any config data variables here }                             
  
function Hash(p : pointer; numb : byte): longint;

procedure Write2Exec(var data; size: word);

implementation

function Hash(p : pointer; numb : byte): longint;
  { Returns a hash of any memory location or variable.     }
  { When originally called numb must be equal to sizeof    }
  { whatever p is pointing at.  If that is a string numb   }
  { should be equal to length(the_string) and p should be  }        
  { ptr(seg(the_string),ofs(the_string)+1)                 }
  var
    temp,
    w    : longint;
    x    : byte;

  begin
    RandSeed := 524149; 
    temp := longint(p^);  RandSeed := temp;
    for x := 0 to (numb - 4) do begin
      w := random(maxint) * random(maxint) * random(maxint);
      temp := ((temp shr random(16)) shl random(16)) +
                w + MemL[seg(p^):ofs(p^)+x];
    end;
    hash := temp;
  end;


procedure InitConstants;
  var
    f           : file;
    tbuff       : array[0..1] of word;
  
  function GetCheckSum : longint;  
    { Performs a checksum calculation on the exe file }
    var
      finished  : boolean;
      x,
      CSum      : longint;
      BytesRead : word;
      buffer    : array[0..4095] of word;
    begin
      {$I-}
      seek(f,0);
      finished := false;  CSum := 0;  x := 0;
      BlockRead(f,buffer,sizeof(buffer),BytesRead);
      while not finished do begin             { do the checksum calculations }
        repeat         { until file has been read up to start of config area }
          inc(CSum,buffer[x mod 4096]);
          inc(x);
          finished := ((x shl 1) >= ExeData.StartConst); 
        until ((x mod 4096) = 0) or finished;
        if not finished then                { data area has not been reached }
          BlockRead(f,buffer,sizeof(buffer),BytesRead);          
      end;
      GetCheckSum := CSum;
    end;
      
  begin
    assign(f, ParamStr(0));
    {$I-} Reset(f,1);
    with ExeData do begin
      if FirstTime and (IOResult = 0) then begin
        Seek(f,2);                   { this location has the executable size }
        BlockRead(f,tbuff,4);
        ExeSize := tbuff[0]+(pred(tbuff[1]) shl 9);
        seek(f,8);                                    {  get the header size }
        BlockRead(f,hsize,2);
        FirstTime := false;
        StartConst := longint(hsize+Seg(ExeData)-PrefixSeg) shl 4 + 
                      Ofs(ExeData) - 256;
        CheckSum := GetCheckSum;
        Seek(f,StartConst);
        BlockWrite(f,ExeData,sizeof(ExeData));
        seek(f,FileSize(f));
        BlockWrite(f,StartConst,4);
      end
      else
        if GetCheckSum <> CheckSum then begin
          writeln('File has been tampered with.  Checksum incorrect');
          halt;
        end;
    end;  { with }    
    Close(f); {$I+}
    if IOResult <> 0 then begin
      writeln('Unable to initialise program');
      halt;
    end;  
  end; { InitConstants }


procedure Write2Exec(var data; size: word);
 { writes a new typed constant into the executable file. }
  var
     f          : file;
  begin
    assign(f, ParamStr(0));
    {$I-} Reset(f,1);
    Seek(f,longint(ExeData.Hsize+Seg(data)-PrefixSeg) shl 4+ Ofs(data)- 256);
    BlockWrite(f,data,size);
    Close(f); {$I+}
    if IOResult <> 0 then;
  end; { Write2Exec }
  

begin
  InitConstants;
end.
